import sqlite3

class Database:
    def __init__(self, db_path: str = 'database.db'):
        connection = sqlite3.connect(db_path)
        self.cursor = connection.cursor()
        self.execute = self.cursor.execute

    def schema(self):
        result = {}

        for table in [row[0] for row in
                      self.execute('SELECT name FROM sqlite_master WHERE type="table" AND name NOT LIKE "sqlite_%"')]:
            result[table] = {}

            for row in self.execute(f'PRAGMA table_info({table})'):
                result[table][row[1]] = row[2]

        return result

    def table(self, table: str, data: dict):
        schema = [row[1] for row in self.execute(f'PRAGMA table_info({table})')]

        if not len(schema):
            self.execute(f'CREATE TABLE {table} (id INTEGER NOT NULL PRIMARY KEY AUTOINCREMENT UNIQUE)')

        for key, value in data.items():
            column_type = 'INTEGER' if isinstance(value, int) else 'REAL' if isinstance(value, float) else 'TEXT'
            if key not in schema: self.execute(f'ALTER TABLE {table} ADD COLUMN {key} {column_type}')

    def drop(self, table: str):
        self.execute(f'DROP TABLE {table}')

    def list(self, table: str, params: dict):
        schema = [row[1] for row in self.execute(f'PRAGMA table_info({table})')]
        where = ' AND '.join([f'{key}="{params[key]}"' for key in params.keys()
                              if key not in ['orderby', 'limit', 'offset']])
        orderby = params['orderby'] if 'orderby' in params else 'id ASC'
        limit = params['limit'] if 'limit' in params else '10'
        offset = params['offset'] if 'offset' in params else '0'

        return [dict(zip(schema, row)) for row in
                self.execute(f'SELECT * FROM {table} WHERE {where if where else "TRUE"} \
                             ORDER BY {orderby} LIMIT {limit} OFFSET {offset}')]

    def create(self, table: str, body: dict):
        keys = ', '.join(list(body.keys()))
        values = str(list(body.values()))[1:-1]
        self.execute(f'INSERT INTO {table} ({keys}) VALUES ({values})')

        return self.read(table, str(self.cursor.lastrowid))

    def read(self, table: str, id: str):
        schema = [row[1] for row in self.execute(f'PRAGMA table_info({table})')]

        return [dict(zip(schema, row)) for row in
                self.execute(f'SELECT * FROM {table} WHERE id = {id} LIMIT 1')][0]

    def update(self, table: str, id: str, body: dict):
        for key, value in body.items(): self.execute(f'UPDATE {table} SET {key} = {value} WHERE id = {id}')

        return self.read(table, id)

    def delete(self, table: str, id: str):
        self.execute(f'DELETE FROM {table} WHERE id = {id}')



from wsgiref import simple_server, util
from urllib.parse import parse_qsl
import os, mimetypes, json

class Server:
    host: str
    port: int
    routes = []

    def __init__(self, port: int = 8000):
        self.host = ''
        self.port = port

    def post(self, route: str):
        return lambda func: self.routes.append(('POST', route, func))

    def get(self, route: str):
        return lambda func: self.routes.append(('GET', route, func))

    def put(self, route: str):
        return lambda func: self.routes.append(('PUT', route, func))

    def delete(self, route: str):
        return lambda func: self.routes.append(('DELETE', route, func))

    def run(self):
        def serve_static(env, res):
            static_folder = 'public' + ('/index.html' if env['PATH_INFO'] == '/' else env['PATH_INFO'])

            if '.' in static_folder and os.path.exists(static_folder):
                res('200 OK', [('Content-Type', mimetypes.guess_type(static_folder)[0])])
                return util.FileWrapper(open(static_folder, "rb"))
            elif env['PATH_INFO'] == '/':
                res('302 Found', [('Location', '/schema.json')])
                return ''
            else:
                res('404 Not Found', [])
                return ''

        def server(env, res):
            path_items = [item for item in env['PATH_INFO'].split('/')[1:] if item]

            for method, route, func in self.routes:
                route_items = [item for item in route.split('/')[1:] if item]

                if method == env['REQUEST_METHOD'] and len(path_items) == len(route_items) and all(
                        [path_items[i] == item or item[0] == '{' for i, item in enumerate(route_items)]):
                    params = [path_items[i] for i, item in enumerate(route_items) if item[0] == '{']
                    data = dict(parse_qsl(env['QUERY_STRING'])) if env['QUERY_STRING'] else {}

                    if env['CONTENT_LENGTH']:
                        body_data = env['wsgi.input'].read(int(env['CONTENT_LENGTH'])).decode()
                        data.update(json.loads(body_data) if env['CONTENT_TYPE'] == 'application/json'
                                    else dict(parse_qsl(body_data)))

                    try:
                        res_body = json.dumps(func(*params, data) if bool(data) else func(*params))
                        res_code = '201 Created' if method == 'POST' else \
                            '204 No Content' if method == 'DELETE' else \
                            '200 OK'
                        res(res_code, [('Content-type', 'application/json; charset=utf-8')])
                        return [res_body.encode()]
                    except: pass

            if env['REQUEST_METHOD'] != 'GET':
                res('400 Bad Request', [])
                return ''
            else:
                return serve_static(env, res)

        with simple_server.make_server(self.host, self.port, server) as httpd:
            print(f'INFO: Application running on {self.host}:{self.port} (Press CTRL+C to quit)')
            httpd.serve_forever()



from sys import argv
from .database import Database
from .server import Server

db = Database(argv[1] if len(argv) > 1 else 'database.db')
app = Server(int(argv[2]) if len(argv) > 2 else 8000)

@app.get('/schema.json')
def schema():
    return db.schema()

@app.get('/{collection}')
def list(collection: str, params: dict = {}):
    return db.list(collection, params)

@app.post('/{collection}')
def create(collection: str, body: dict):
    db.table(collection, body)
    return db.create(collection, body)

@app.delete('/{collection}')
def drop(collection: str):
    return db.drop(collection)

@app.get('/{collection}/{id}')
def read(collection: str, id: str):
    return db.read(collection, id)

@app.put('/{collection}/{id}')
def update(collection: str, id: str, body: dict):
    return db.update(collection, id, body)

@app.delete('/{collection}/{id}')
def delete(collection: str, id: str):
    db.delete(collection, id)

def main():
    app.run()

if __name__ == '__main__':
    main()



from setuptools import setup, find_packages

with open("README.md", "r", encoding="utf-8") as fh:
    long_description=fh.read()

setup(
    name="Rest-In-Peace",
    version="0.1.2",
    author="Henrique Gogó",
    author_email="henriquegogo@gmail.com",
    description="An instant and schemaless rest api with sqlite",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/henriquegogo/rest_in_peace",
    license="MIT",
    project_urls={
        "Bug Tracker": "https://github.com/henriquegogo/rest_in_peace/issues"
    },
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    package_dir={"": "src"},
    packages=find_packages(where="src"),
    python_requires=">=3.6",
    entry_points={
        "console_scripts": ["rest-in-peace=rest_in_peace.cli:main"]
    }
)
